"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const index_1 = require("./index");
const NOMAP = () => { };
function arrayFrom(arg, tree) {
    var map;
    if (core_1.isUndefined(tree)) {
        tree = arg;
        map = NOMAP;
    }
    else {
        map = arg;
    }
    var array = new Array(index_1.size(tree));
    var it = index_1.iterateFromFirst(tree);
    var i, entry;
    if (map === NOMAP) {
        for (i = 0; i < array.length; i++) {
            array[i] = it.next().value;
        }
    }
    else {
        for (i = 0; i < array.length; i++) {
            entry = it.next().value;
            array[i] = map(entry.value, entry.key, i);
        }
    }
    return array;
}
exports.arrayFrom = arrayFrom;
/**
 * Returns a value iterator; one for each entry in the tree. The iterator is guaranteed to iterate in the same order as
 * the corresponding entries in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTreeStructure<K, V>} tree The tree to read values from
 * @returns {IterableIterator<V>} An iterable iterator that will visit each value in the tree
 */
function values(tree) {
    return index_1.iterateValuesFromFirst(tree);
}
exports.values = values;
/**
 * Returns a key iterator; one for each entry in the tree. The iterator is guaranteed to iterate in the same order as
 * the corresponding entries in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTreeStructure<K, V>} tree The tree to read values from
 * @returns {IterableIterator<K>} An iterable iterator that will visit each key in the tree
 */
function keys(tree) {
    return index_1.iterateKeysFromFirst(tree);
}
exports.keys = keys;

//# sourceMappingURL=unwrap.js.map
