"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const node_1 = require("./node");
const path_1 = require("./path");
const find_1 = require("./find");
class RedBlackTreeIterator {
    constructor(_current, _compare, _reversed) {
        this._current = _current;
        this._compare = _compare;
        this._reversed = _reversed;
        this._begun = false;
    }
    static create(current, compare, reversed) {
        if (current.isActive()) {
            current.next = reversed ? 2 /* RIGHT */ : 1 /* LEFT */; // indicates the already-traversed branch
        }
        return new RedBlackTreeIterator(current, compare, !!reversed);
    }
    _next(reverse) {
        var current = this._current;
        var result = { done: false, value: node_1.NONE };
        if (current.isNone()) {
            result.done = true;
            return result;
        }
        if (!this._begun) {
            this._begun = true;
            result.value = current.node;
            return result;
        }
        var done = false, node, canEmit = false;
        do {
            switch (current.next) {
                case 0 /* NONE */:
                    if (reverse) {
                        node = current.node._right;
                        current.next = 2 /* RIGHT */;
                    }
                    else {
                        node = current.node._left;
                        current.next = 1 /* LEFT */;
                    }
                    if (!node_1.isNone(node)) {
                        current = path_1.PathNode.next(node, current, 0 /* NONE */);
                    }
                    break;
                case 1 /* LEFT */:
                    if (reverse) {
                        current = current.release();
                    }
                    else if (canEmit) {
                        result.value = current.node;
                        done = true;
                    }
                    else {
                        node = current.node._right;
                        if (node_1.isNone(node)) {
                            current = current.release();
                        }
                        else {
                            current.next = 2 /* RIGHT */;
                            current = path_1.PathNode.next(node, current, 0 /* NONE */);
                        }
                    }
                    break;
                case 2 /* RIGHT */:
                    if (!reverse) {
                        current = current.release();
                    }
                    else if (canEmit) {
                        result.value = current.node;
                        done = true;
                    }
                    else {
                        node = current.node._left;
                        if (node_1.isNone(node)) {
                            current = current.release();
                        }
                        else {
                            current.next = 1 /* LEFT */;
                            current = path_1.PathNode.next(node, current, 0 /* NONE */);
                        }
                    }
                    break;
            }
            if (current.isNone()) {
                result.done = true;
                result.value = node_1.NONE;
                done = true;
            }
            else {
                canEmit = true;
            }
        } while (!done);
        this._current = current;
        return result;
    }
    _findNext(reverse, inclusive, key) {
        const result = { done: true, value: node_1.NONE };
        let current = this._current, c;
        if (current.isActive()) {
            if ((!this._begun && (this._begun = true, c = this._compare(current.node.key, key), c = reverse ? -c : c, inclusive ? c >= 0 : c > 0)) ||
                (this._current = current = core_1.valueOrDefault(find_1.findNext(this._compare, reverse, inclusive, key, current), path_1.PathNode.NONE), current.isActive())) {
                result.done = false;
                result.value = current.node;
                current.next = reverse ? 2 /* RIGHT */ : 1 /* LEFT */;
            }
        }
        return result;
    }
    next(key, inclusive) {
        return core_1.isUndefined(key) ? this._next(this._reversed) : this._findNext(this._reversed, inclusive !== false, key);
    }
    previous(key, inclusive) {
        return core_1.isUndefined(key) ? this._next(!this._reversed) : this._findNext(!this._reversed, inclusive !== false, key);
    }
    clone() {
        return new RedBlackTreeIterator(path_1.clonePath(this._current), this._compare, this._reversed);
    }
    [Symbol.iterator]() {
        return this;
    }
}
exports.RedBlackTreeIterator = RedBlackTreeIterator;
class RedBlackTreeKeyIterator {
    constructor(_it) {
        this._it = _it;
    }
    next(key, inclusive) {
        const result = this._it.next(key, inclusive);
        return (result.value = result.value.key, result);
    }
    previous(key, inclusive) {
        const result = this._it.previous(key, inclusive);
        return (result.value = result.value.key, result);
    }
    clone() {
        return new RedBlackTreeKeyIterator(this._it.clone());
    }
    [Symbol.iterator]() {
        return this;
    }
}
exports.RedBlackTreeKeyIterator = RedBlackTreeKeyIterator;
class RedBlackTreeValueIterator {
    constructor(_it) {
        this._it = _it;
    }
    next(key, inclusive) {
        const result = this._it.next(key, inclusive);
        return (result.value = result.value.value, result);
    }
    previous(key, inclusive) {
        const result = this._it.previous(key, inclusive);
        return (result.value = result.value.value, result);
    }
    clone() {
        return new RedBlackTreeValueIterator(this._it.clone());
    }
    [Symbol.iterator]() {
        return this;
    }
}
exports.RedBlackTreeValueIterator = RedBlackTreeValueIterator;

//# sourceMappingURL=iterator.js.map
