"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_1 = require("./node");
function writeBack(upper, lower, tree) {
    if (upper.isActive()) {
        setChild(upper.next, upper.node, lower);
    }
    else {
        tree._root = lower;
    }
}
function rotateLeft(upper, parent, child, tree) {
    if (node_1.isNone(parent))
        throw new Error('rotateLeft: Parent is NIL'); // ## DEV
    if (node_1.isNone(child))
        throw new Error('rotateLeft: Child is NIL'); // ## DEV
    parent._right = child._left;
    child._left = parent;
    updateCount(parent);
    updateCount(child);
    writeBack(upper, child, tree);
    return child;
}
exports.rotateLeft = rotateLeft;
function rotateRight(upper, parent, child, tree) {
    if (node_1.isNone(parent))
        throw new Error('rotateRight: Parent is NIL'); // ## DEV
    if (node_1.isNone(child))
        throw new Error('rotateRight: Child is NIL'); // ## DEV
    parent._left = child._right;
    child._right = parent;
    updateCount(parent);
    updateCount(child);
    writeBack(upper, child, tree);
    return child;
}
exports.rotateRight = rotateRight;
function rotateLeftRight(upper, grandParent, parent, child, tree) {
    parent._right = child._left;
    child._left = parent;
    grandParent._left = child._right;
    child._right = grandParent;
    updateCount(grandParent);
    updateCount(parent);
    updateCount(child);
    writeBack(upper, child, tree);
    return child;
}
exports.rotateLeftRight = rotateLeftRight;
function rotateRightLeft(upper, grandParent, parent, child, tree) {
    parent._left = child._right;
    child._right = parent;
    grandParent._right = child._left;
    child._left = grandParent;
    updateCount(grandParent);
    updateCount(parent);
    updateCount(child);
    writeBack(upper, child, tree);
    return child;
}
exports.rotateRightLeft = rotateRightLeft;
function swapNodeContents(upper, lower) {
    var key = upper.key;
    var value = upper.value;
    upper.key = lower.key;
    upper.value = lower.value;
    lower.key = key;
    lower.value = value;
}
exports.swapNodeContents = swapNodeContents;
function swapNodeColors(upper, lower) {
    var red = upper._red;
    upper._red = lower._red;
    lower._red = red;
}
exports.swapNodeColors = swapNodeColors;
function setChild(branch, parent, child) {
    if (branch === 0 /* NONE */)
        return;
    if (branch === 1 /* LEFT */) {
        parent._left = child;
    }
    else {
        parent._right = child;
    }
    updateCount(parent);
}
exports.setChild = setChild;
function updateCount(node) {
    if (node_1.isNone(node))
        return;
    node._count = node._left._count + node._right._count + 1;
}
exports.updateCount = updateCount;

//# sourceMappingURL=ops.js.map
