"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const node_1 = require("./node");
const ops_1 = require("./ops");
class PathNode {
    constructor(node, parent, next) {
        this.node = node;
        this.parent = parent;
        this.next = next;
    }
    static next(node, parent, next) {
        var p = PathNode.cache;
        if (p.isActive()) {
            PathNode.cache = p.parent;
            p.node = node;
            p.parent = parent;
            p.next = next;
        }
        else {
            p = new PathNode(node, parent, next);
        }
        return p;
    }
    static release(p, node) {
        if (core_1.isUndefined(node)) {
            node = node_1.NONE;
        }
        do {
            p.node = node_1.NONE;
        } while (p.parent.isActive() && (p = p.parent, node = p.node));
        p.parent = PathNode.cache;
        return node;
    }
    static releaseAndRecount(p, node) {
        do {
            ops_1.updateCount(p.node);
            p.node = node_1.NONE;
        } while (p.parent.isActive() && (p = p.parent, node = p.node));
        p.parent = PathNode.cache;
        return node;
    }
    isActive() {
        return this !== PathNode.NONE;
    }
    isNone() {
        return this === PathNode.NONE;
    }
    replace(node) {
        if (this.parent.isActive()) {
            ops_1.setChild(this.parent.next, this.parent.node, node);
        }
        this.node = node;
    }
    release() {
        var p = this.parent;
        this.node = node_1.NONE;
        this.parent = PathNode.cache;
        PathNode.cache = this;
        return p;
    }
}
exports.PathNode = PathNode;
PathNode.NONE = new PathNode(node_1.NONE, void 0, 0 /* NONE */);
PathNode.NONE.parent = PathNode.NONE;
PathNode.cache = PathNode.NONE;
function findPath(tree, key, root, compare, p) {
    var node = root; // Assumes root has already been assigned. Check for a void root before calling insert().
    if (core_1.isUndefined(p)) {
        p = PathNode.NONE;
    }
    do {
        var c = compare(key, node.key);
        if (c < 0) {
            p = PathNode.next(node, p, 1 /* LEFT */);
            node = node_1.editLeftChild(tree, node);
        }
        else if (c > 0) {
            p = PathNode.next(node, p, 2 /* RIGHT */);
            node = node_1.editRightChild(tree, node);
        }
        else {
            p = PathNode.next(node, p, 0 /* NONE */);
            node = node_1.NONE;
        }
    } while (!node_1.isNone(node));
    return p;
}
exports.findPath = findPath;
function findSuccessor(tree, p) {
    p.next = 2 /* RIGHT */;
    var node = node_1.editRightChild(tree, p.node);
    while (!node_1.isNone(node._left)) {
        p = PathNode.next(node, p, 1 /* LEFT */);
        node = node_1.editLeftChild(tree, node);
    }
    p = PathNode.next(node, p, 0 /* NONE */);
    return p;
}
exports.findSuccessor = findSuccessor;
function clonePath(path) {
    let child = path = PathNode.next(path.node, path.parent, path.next), parent = child;
    while (!parent.isNone()) {
        child.parent = parent = PathNode.next(parent.node, parent.parent, parent.next);
        child = parent;
        parent = child.parent;
    }
    return path;
}
exports.clonePath = clonePath;

//# sourceMappingURL=path.js.map
