import { Mutation, ComparatorFn, Associative } from '@collectable/core';
import { IndexedCollection } from '@collectable/core';
import { Node, RedBlackTreeEntry } from './node';
import { RedBlackTreeIterator } from './iterator';
export declare class RedBlackTreeStructure<K, V = null> implements IndexedCollection<K, V, RedBlackTreeEntry<K, V>, Associative<V>> {
    /** @internal */
    _compare: ComparatorFn<K>;
    /** @internal */
    _root: Node<K, V>;
    /** @internal */
    _size: number;
    /** @internal */
    constructor(mctx: Mutation.Context, compare: ComparatorFn<K>, root: Node<K, V>, size: number);
    /** @internal */
    readonly '@@mctx': Mutation.Context;
    /** @internal */
    readonly '@@is-collection': true;
    /** @internal */
    readonly '@@size': number;
    /** @internal */
    '@@clone'(mctx: Mutation.Context): RedBlackTreeStructure<K, V>;
    /** @internal */
    '@@equals'(other: RedBlackTreeStructure<K, V>): boolean;
    /** @internal */
    '@@hash'(): number;
    /** @internal */
    '@@unwrap'(): any;
    /** @internal */
    '@@unwrapInto'(target: Associative<V>): Associative<V>;
    /** @internal */
    '@@createUnwrapTarget'(): Associative<V>;
    /** @internal */
    '@@get'(key: K): V | undefined;
    /** @internal */
    '@@has'(key: K): boolean;
    /** @internal */
    '@@set'(key: K, value: V): this;
    /** @internal */
    '@@update'(updater: (value: V, tree: this) => any, key: K): this;
    /** @internal */
    '@@verifyKey'(key: any): boolean;
    [Symbol.iterator](): RedBlackTreeIterator<K, V>;
}
export declare function isRedBlackTree<K, V = any>(arg: any): arg is RedBlackTreeStructure<K, V>;
export declare function createTree<K, V>(compare: ComparatorFn<K>, mutability?: Mutation.PreferredContext): RedBlackTreeStructure<K, V>;
export declare function cloneTree<K, V>(tree: RedBlackTreeStructure<K, V>, mutability?: Mutation.PreferredContext): RedBlackTreeStructure<K, V>;
